package
{
	import org.flixel.*;

	public class Game
	{
		
		[Embed(source = "data/p1.png")] public static var ImgP1:Class;
		[Embed(source = "data/p2.png")] public static var ImgP2:Class;
		
		public static const width:int=10;
		public static const height:int=10;
		public static const p1cola:uint=0xff55ffe5;
		public static const p1colb:uint=0xff00ff98;
		public static const p2cola:uint=0xffff6e55;
		public static const p2colb:uint=0xffff001a;
		public static const pnonecola:uint=0xffaaaaaa;
		public static const pnonecolb:uint=0xff7f7f7f;
		public static const bgcolor:uint=0xff3d2509;
		public static const tilesize:int=5;
		public static const tileborder:int=1;
		public static const xmargin:int=1;
		public static const ymargin:int=1;
		
		public static var tilecols:Array = [
				[pnonecola,pnonecolb], //unoccupied
				[p1cola,p1colb],
				[p2cola,p2colb]
			];
		
		public var board:Array;
		public var p2x:int = width/2 ;
		public var p2y:int = 2;
		public var p1x:int = width-1-p2x;
		public var p1y:int = height-1-p2y;
		
		public var rating:int=0;
		public var winner:int=0;
		
		public static function NewBoard():Game
		{
			var result:Game = new Game();
			/*
			for (var i:int=0;i<20;i++)
			{
				result.board[int(Math.random()*height)][int(Math.random()*width)]=int(Math.random()*2)+1;
			}
			*/
			result.board[result.p1y][result.p1x-1]=1;
			result.board[result.p1y][result.p1x+1]=1;
			result.board[result.p1y+1][result.p1x]=1;
			result.board[result.p1y-1][result.p1x]=1;			
			result.board[result.p1y][result.p1x]=1;
			result.board[result.p2y][result.p2x-1]=2;
			result.board[result.p2y][result.p2x+1]=2;
			result.board[result.p2y+1][result.p2x]=2;
			result.board[result.p2y-1][result.p2x]=2;			
			result.board[result.p2y][result.p2x]=2;
			result.rating = 0;
			return result;			
		}
		
		public function FindBestAIMove():void
		{
			
		}
		
		public function Game()
		{
			board=[];
			for (var j:int=0;j<height;j++)
			{
				var row:Array=[];
				for (var i:int=0;i<width;i++)
				{
					row.push(0);
				}
				board.push(row);
			}			
		}
		
		//player is either 1 or 2
		public function CanMove(player:int,dx:int,dy:int):Boolean
		{
			var px:int= (player==1?p1x:p2x)+dx;
			var py:int= (player==1?p1y:p2y)+dy;
			if (px<0 || px>=width || py<0 ||py>=height)
			{
				return false;
			}
			else
			{
				var boardnum:int = board[py][px];
				return boardnum==player;
			} 
		}
		
		public function GetPiece(x:int,y:int):int
		{
			if (x<0 || x>=width || y<0 ||y>=height)
				return -1;
			else
				return board[y][x];
		}
		
		public function Copy():Game
		{
			var g:Game = new Game();
			for (var i:int=0;i<width;i++)
			{
				for (var j:int=0;j<height;j++)
				{
					g.board[j][i]=board[j][i];
				}
			}
			g.p1x=p1x;
			g.p1y=p1y;
			g.p2x=p2x;
			g.p2y=p2y;
			g.label=label;
			g.rating=rating;
			return g;
		}
		
		//assumes move possible
		public function DoMove(player:int,dx:int,dy:int):Game
		{
			var g:Game = Copy();
			if (player==1)
			{
				g.p1x+=dx;
				g.p1y+=dy;
			}
			else
			{
				g.p2x+=dx;
				g.p2y+=dy;
			}
			
			for (var i:int=0;i<width;i++)
			{
				for (var j:int=0;j<height;j++)
				{
					if (board[j][i]==player)
					{
						var targetpiece:int = GetPiece(i+dx,j+dy);
						if (targetpiece==0)
						{
								g.board[j+dy][i+dx]=player;
								g.rating+=player*2-3;
						}
						else if (targetpiece==3-player)//3-player maps 1 to 2 and 2 to 1
						{
							g.board[j+dy][i+dx]=0;
							g.rating-=player*2-3;							
						}
					}
				}
			}
			
			//CHECK FOR WIN STATE HERE
			var otherx:int=player==1?p2x:p1x;
			var othery:int=player==1?p2y:p1y;
			if (g.board[othery][otherx]==0)
			{
				g.winner=player;
//				trace("winner set to be "+g.winner);
			}
			return g;
		}
		
		public function DrawBoard():FlxGroup
		{
			var result:FlxGroup=new FlxGroup();
			for (var j:int=0;j<height;j++)
			{
				for (var i:int=0;i<width;i++)
				{
					var tiletype:int = board[j][i];
					var tilecol:int = tilecols[tiletype][(i+j)%2];
					var s:FlxSprite;
					s = new FlxSprite(xmargin+(tilesize+tileborder)*i,ymargin+(tilesize+tileborder)*j);
					s.createGraphic(tilesize,tilesize,tilecol);
					result.add(s);
				}
			}	
			
			s = new FlxSprite(xmargin+(tilesize+tileborder)*p1x,ymargin+(tilesize+tileborder)*p1y,ImgP1);
			result.add(s);
			s = new FlxSprite(xmargin+(tilesize+tileborder)*p2x,ymargin+(tilesize+tileborder)*p2y,ImgP2);
			result.add(s);
			
			return result;
		}
		
		public static var dirs:Array=[
			[0,-1],
			[0,1],
			[-1,0],
			[1,0]];
		
		public var label:int=-1;
		
		public function GenerateMovesOfDepth(depth:int, _label:int):Array
		{
			//if label is -1, lavel with move num
			var result:Array = [];
			for (var i:int=0;i<4;i++)
			{
				var dx:int=dirs[i][0];
				var dy:int=dirs[i][1];
				if (CanMove(2,dx,dy))
				{
					var g:Game = DoMove(2,dx,dy);
					if (_label==-1)
					{
						g.label=i;
					}
					//check for p2 win here
					if (g.winner==2)
					{
						result.push(g);
					}
					else
					{
						//for winning state, just append winning state, for p1 win states, don't bother adding
						for (var j:int=0;j<4;j++)
						{
							var dx2:int=dirs[j][0];
							var dy2:int=dirs[j][1];
							if (g.CanMove(1,dx2,dy2))
							{
								var g2:Game = g.DoMove(1,dx2,dy2);
								if (g2.winner==1)
								{
									result.push(g2);
								}
								else
								{
									if (depth>1)
									{
										var toadd:Array = g2.GenerateMovesOfDepth(depth-1,g.label);
										for (var ti:int=0;ti<toadd.length;ti++)
										{
											result.push(toadd[ti]);
										}
									}
									else
									{
										result.push(g2);
									}
								}
							}
						}
					}
				}
			}
			
			return result;
		}	
		
		
		public function BestAIMove(depth:int):int
		{		
			//first, check if any depth 1 moves lead anywhere:
			cands = GenerateMovesOfDepth(1,-1);
			for (i=0;i<cands.length;i++)
			{
				g = cands[i];
				if (g.winner==2)
				{
					//trace("seize victory");
					return g.label;
				}
			}
			
			var cands:Array = GenerateMovesOfDepth(depth,-1);
		//	shuffleKnuth(cands);
			var set:Array=[false,false,false,false];
			var mins:Array=[10000,10000,10000,10000];
			var canwin:Array=[false,false,false,false];
			
			for (var i:int=0;i<cands.length;i++)
			{
				var g:Game = cands[i];
				if (g.winner==1)
				{
					mins[g.label]=-10000;
					set[g.label]=true;
				}
				else if (g.rating<mins[g.label])
				{
					mins[g.label]=g.rating;
					set[g.label]=true;
				}
			}
			
			var max:int=-10000;
			var maxint:int=-1;
			for (i=0;i<4;i++)
			{
				if (set[i])
				{
					if (mins[i]>max)
					{
						max=mins[i];
						maxint=i;
					}
				}
			}
			
			if (maxint==-1)
			{
				//trace("no good moves:(");
				maxint = cands[0].label;
			}
			
			return maxint;
			/*
			var bestscore:int=-10000;
			var bestdir:int=-1;
			//trace("candslength " + cands.length);
			for (var i:int=0;i<cands.length;i++)
			{
				var g:Game = cands[i];
				if (g.winner==2)
				{
					trace("found winning move in direction " + g.label);
					bestdir=g.label;
					bestscore=1000000;
					break;
				}
				else if (g.rating>bestscore)
				{
					bestscore=g.rating;
					bestdir=g.label;
				}
			}
			
			return bestdir;
			*/
			/*for (var i:int=0;i<4;i++)
			{
				var dx:int=dirs[i][0];
				var dy:int=dirs[i][1];
				if (CanMove(2,dx,dy))
				{
					var g:Game = DoMove(2,dx,dy);
					
					for (var j:int=0;j<4;j++)
					{
						var dx2:int=dirs[j][0];
						var dy2:int=dirs[j][1];
						if (g.CanMove(1,dx2,dy2))
						{
							var g2:Game = g.DoMove(1,dx2,dy2);
							if (g2.rating>bestscore)
							{
								bestscore=g2.rating;
								bestdir=i;
							}
							else if (g2.winner==2)
							{
								bestscore=1000000;
								bestdir=i;
							}
						}
					}
				}
			}
			trace("best = "+bestscore);
			return bestdir;*/
		}
	}
}